package xyz.mtfos.btdemo

import android.Manifest
import android.annotation.SuppressLint
import android.app.Activity
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.text.Editable
import android.widget.*
import com.google.zxing.integration.android.IntentIntegrator
import com.google.zxing.integration.android.IntentResult
import org.json.JSONObject
import xyz.mtfos.btdemo.objectTool.bind

/**
 * Created by jay on 2017/8/9.
 */
class MainActivity : Activity() {
    val mainLay: LinearLayout by bind(R.id.mainlay)

    val intxt: EditText by bind(R.id.intxt)
    val btn: Button by bind(R.id.bt)
    val connectBtn: Button by bind(R.id.connect)
    val disconnectBtn: Button by bind(R.id.disconnect)
    var ble: PrinterBle? = null
    val alignSpinner: Spinner by bind(R.id.align)
    val sizeSpinner: Spinner by bind(R.id.size)
    val alignBtn: Button by bind(R.id.btn_align)
    val sizeBtn: Button by bind(R.id.btn_size)

    val inmac: EditText by bind(R.id.inmac)
    val inservice: EditText by bind(R.id.inservice)
    val incharacteristic: EditText by bind(R.id.incharacteristic)
    val scanSetting: Button by bind(R.id.scansetting)

    val addHR: Button by bind(R.id.addHR)
    val addNL: Button by bind(R.id.addNL)
    val addTAB: Button by bind(R.id.addTAB)
    val addTXT: Button by bind(R.id.addTXT)

    val preview: TextView by bind(R.id.preview)

    val uiHandler: Handler = Handler()
    var th: Thread? = null
    var thrun: Boolean = false
    var isInit: Boolean = false
    val alignArr: Array<String> = arrayOf("Left", "Center", "Right")
    val sizeArr: Array<String> = arrayOf("1,1", "1,2", "2,1", "2,2")

    @SuppressLint("WrongConstant")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        mainLay.visibility = LinearLayout.GONE

        btn.isEnabled = false
        connectBtn.isEnabled = true
        disconnectBtn.isEnabled = false

        ble = PrinterBle(this@MainActivity)

        ble?.setOnStateChangeListener {
            System.out.println("Now State is >>>> " + it)
            if (it == PrinterBle.CONNECTING) {
                disconnectBtn.isEnabled = true
            }
            if (it == PrinterBle.CONNECTED) {
                btn.isEnabled = true
                connectBtn.isEnabled = false
                disconnectBtn.isEnabled = true
                uiHandler.post {
                    mainLay.visibility = LinearLayout.VISIBLE
                }
            }
            if (it == PrinterBle.DISCONNECTED) {
                btn.isEnabled = false
                connectBtn.isEnabled = true
                disconnectBtn.isEnabled = false
                uiHandler.post {
                    mainLay.visibility = LinearLayout.GONE
                }
            }
        }

        scanSetting.setOnClickListener {
            IntentIntegrator(this@MainActivity).initiateScan()
        }

        ble?.setOnNotifyListener {
            System.out.println("Get Notify :::::: " + it)
        }

        alignSpinner.adapter = ArrayAdapter<String>(this@MainActivity, android.R.layout.simple_spinner_item, alignArr)
        alignBtn.setOnClickListener {
            val pos = alignSpinner.selectedItemPosition
            when (pos) {
                0 -> ble?.setAlign(PrinterBle.ALIGN_LEFT)
                1 -> ble?.setAlign(PrinterBle.ALIGN_CENTER)
                2 -> ble?.setAlign(PrinterBle.ALIGN_RIGHT)
            }
            refreshPreview()
        }

        sizeSpinner.adapter = ArrayAdapter<String>(this@MainActivity, android.R.layout.simple_spinner_item, sizeArr)
        sizeBtn.setOnClickListener {
            val pos = sizeSpinner.selectedItemPosition
            when (pos) {
                0 -> ble?.setSize(1, 1)
                1 -> ble?.setSize(1, 2)
                2 -> ble?.setSize(2, 1)
                3 -> ble?.setSize(2, 2)
            }
            refreshPreview()
        }

        var permissionCheck: Int = 0
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            permissionCheck += this@MainActivity.checkSelfPermission(android.Manifest.permission.ACCESS_COARSE_LOCATION)
            permissionCheck += this@MainActivity.checkSelfPermission(android.Manifest.permission.ACCESS_FINE_LOCATION)
            permissionCheck += this@MainActivity.checkSelfPermission(android.Manifest.permission.CAMERA)
            if (permissionCheck != 0) {
                requestPermissions(arrayOf(Manifest.permission.ACCESS_COARSE_LOCATION, Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.CAMERA), 111)
            }
        }

        addHR.setOnClickListener {
            ble?.addHR()
            refreshPreview()
        }

        addNL.setOnClickListener {
            ble?.textNewLine()
            refreshPreview()
        }

        addTAB.setOnClickListener {
            ble?.textTab()
            refreshPreview()
        }

        addTXT.setOnClickListener {
            val txt: String = intxt.text.toString()
            ble?.addText(txt)
            refreshPreview()
        }

        btn.setOnClickListener {
            ble?.sendQueue()
        }

        connectBtn.setOnClickListener {
            ble?.macAddr = inmac.text.toString()
            ble?.serviceUUID = inservice.text.toString()
            ble?.characteristicUUID = incharacteristic.text.toString()
            scanConnect()
        }
        disconnectBtn.setOnClickListener {
            disconnect()
        }
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>?, grantResults: IntArray?) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        val qrRes: IntentResult? = IntentIntegrator.parseActivityResult(requestCode, resultCode, data)
        if (qrRes != null) {
            val content: String? = qrRes?.contents
            if (content != null) {
                System.out.println("GET QRCODE :: $content")
                try {
                    val json: JSONObject = JSONObject(content)
                     if(json.has("mac") && json["mac"] is String){
                         inmac.text = Editable.Factory.getInstance().newEditable(json.getString("mac"))
                     }
                    if(json.has("service") && json["service"] is String) {
                        inservice.text = Editable.Factory.getInstance().newEditable(json.getString("service"))
                    }
                    if(json.has("characteristic") && json["characteristic"] is String) {
                        incharacteristic.text = Editable.Factory.getInstance().newEditable(json.getString("characteristic"))
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                    Toast.makeText(this@MainActivity, "Parser Error", Toast.LENGTH_SHORT).show()
                }
            }
        }
    }

    override fun onStop() {
        super.onStop()
        disconnect()
    }

    fun scanConnect() {
        if (ble?.init()!!) {
            ble?.startScan()
        }
    }

    fun disconnect() {
        if (ble?.mState!! == PrinterBle.SCANNING) {
            ble?.stopScan()
        }
        if (ble?.mState!! == PrinterBle.CONNECTED) {
            ble?.disconnect()
        }
    }

    fun refreshPreview() {
        val data = ble?.dataQueue ?: return
        preview.text = ""
        for (str: String in data) {
            preview.text = preview.text.toString() + "\n" + str
        }
    }
}